import requests
from . import utils

__all__ = ["URL"]


if requests.compat.is_py2:
    from urlparse import parse_qs
elif requests.compat.is_py3:
    from urllib.parse import parse_qs


class URL(object):
    def __init__(self, url_str):
        """parse url query part into dict

        :param url: url to parse

        Usage::

            >>> params = parse_query("http://www.example.com/?foo=bar&baz")
            {'foo': 'bar', 'baz': None}

        """
        parsed_url = requests.compat.urlparse(utils.to_str(url_str))
        netloc_parts = parsed_url.netloc.split("@")
        if len(netloc_parts) == 1:
            username = password = None
            host_str = netloc_parts[0]
        else:
            username, password = netloc_parts[0].split(":")
            host_str = netloc_parts[1]

        host_parts = host_str.split(":")
        host = host_parts[0]

        if len(host_parts) == 1:
            port = 80
        else:
            port = int(host_parts[1])

        params = [
            (key, val[0] if val[0] else None)
            for key, val in parse_qs(parsed_url.query, True).items()
        ]

        self._info = dict(
            scheme=parsed_url.scheme or "http",
            username=username,
            password=password,
            host=host,
            port=port,
            path=parsed_url.path or "/",
            params=params,
            fragment=parsed_url.fragment
        )
        self._url = None

    def forge(self, **kwargs):
        if kwargs:
            self._info["params"].sort(**kwargs)

        parts = [
            "{0}://".format(self.scheme),
            self.netloc,
            self.uri
        ]
        self._url = utils.to_str("".join(parts))
        return self._url

    def __str__(self):
        if self._url is None:
            self.forge()
        return self._url

    @property
    def scheme(self):
        return self._info["scheme"]

    @property
    def netloc(self):
        if self.username is None or self.password is None:
            netloc = self.host
        else:
            netloc = "{0}:{1}@{2}".format(
                self.username,
                self.password,
                self.host
            )
        if self.port != 80:
            netloc = "{0}:{1}".format(netloc, self.port)
        return netloc

    @property
    def username(self):
        return self._info["username"]

    @property
    def password(self):
        return self._info["password"]

    @property
    def host(self):
        return self._info["host"]

    @property
    def port(self):
        return self._info["port"]

    @property
    def path(self):
        return self._info["path"]

    @path.setter
    def path(self, new_val):
        self._info["path"] = new_val
        return new_val

    @property
    def params(self):
        return dict(self._info["params"])

    @params.setter
    def params(self, new_params):
        if isinstance(new_params, dict):
            self._info["params"] = list(new_params.items())
        else:
            self._info["params"] = list(new_params)
        return new_params

    @property
    def query(self):
        query = []
        for key, val in self._info["params"]:
            if val is None:
                query.append(requests.compat.quote(utils.to_bytes(key)))
            else:
                param = {utils.to_bytes(key): utils.to_bytes(val)}
                query.append(requests.compat.urlencode(param))
        return "&".join(query)

    @property
    def uri(self):
        query = self.query
        fragment = self.fragment
        return "".join([
            self.path,
            "?{0}".format(query) if query else "",
            "#{0}".format(fragment) if fragment else ""
        ])

    def append_params(self, new_params):
        params = dict(self.params, **dict(new_params))
        self.params = params
        return self.params

    @property
    def fragment(self):
        return self._info["fragment"]



# -*- coding:utf8 -*-

"""
aliyunauth.rds
~~~~~~~~~~~~~~

This module contains the authentication handlers for Ali Rds Service
"""

from . import sign_ver_1_0


class RdsAuth(sign_ver_1_0.AuthBase):
    """Attach Aliyun Rds Authentication to the given request

    :param access_key: the access_key of your ecs account
    :param secret_key: the secret_key of your ecs account
    :param response_format: (optional) response format [`xml`/`json`(default)]
    :param ram: (optional) resource access managment string (default None)

    Usage::

        >>> import requests
        >>> from aliyunauth import RdsAuth
        >>> req = request.get(
        ...     "https://rds.aliyuncs.com",
        ...     params=dict(Action="DescribeInstanceTypes"),
        ...     auth=EcsAuth("access-key", "secret-key")
        ... )
        <Response [200]>

    """
    SERVICE = "rds"
    VERSION = "2014-08-15"



# -*- coding:utf8 -*-

import base64
import hashlib
import os

import requests

from . import consts


def cal_b64md5(data):
    """

    :param data: cal md5 checksum for this piece data

    Usage::

        >>> cal_b64md5(io.BytesIO(b"hello world"))
        XrY7u+Ae7tCTyyK7j1rNww==

    """
    if data is None:
        return None
    md5sum = hashlib.md5()
    if hasattr(data, "read") and hasattr(data, "seek"):
        # file-like object or IO-like object
        data_piece = data.read(consts.MD5_CHUNK_SIZE)
        while data_piece:
            md5sum.update(to_bytes(data_piece))
            data_piece = data.read(consts.MD5_CHUNK_SIZE)
        data.seek(0, os.SEEK_SET)
    else:
        md5sum.update(to_bytes(data))
    return to_str(base64.b64encode(md5sum.digest()))


def to_bytes(some_str, encoding="utf8"):
    if isinstance(some_str, requests.compat.str):
        some_bytes = some_str.encode(encoding)
    else:
        some_bytes = some_str
    return some_bytes


def to_str(some_bytes, encoding="utf8"):
    if isinstance(some_bytes, requests.compat.bytes):
        some_str = some_bytes.decode(encoding)
    else:
        some_str = some_bytes
    return some_str


def percent_quote(query):
    return requests.compat.quote(to_bytes(query), consts.PERCENT_SAFE)


def percent_encode(params_tuple, sort=False):
    if sort:
        params = sorted(params_tuple, key=lambda x: x[0])
    else:
        params = list(params_tuple)

    encoded = "&".join([
        "{0}={1}".format(percent_quote(opt), percent_quote(val))
        for opt, val in params
        if val is not None
    ])
    return encoded
